//
//  FLTK test program for:
//
//  STR #3188: Callback added using Fl::add_timeout is not called (X11)
//
//  Compile:
//
//	fltk-config --compile str-3188_add_timeout.cxx
//
//  Expected behavior:
//
//	The "Hello World!" box should change its background color
//	every two seconds.
//
//  Old behavior w/o patch, i.e. svn current (r11914) as of 09/02/2016:
//
//	Background color changes only if other events are processed,
//	for instance moving the mouse over the window.
//
//  New behavior with patch:
//
//	Background color changes as expected every two seconds.
//

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>

#include <stdio.h>

static void timer_cb(void *v) {
  static Fl_Color c = 0;
  Fl::remove_timeout(timer_cb); // make sure to _remove_ the timeout
  Fl_Widget *w = (Fl_Widget*)v;
  w->color(c++);
  if (c > 255) c = 0;
  w->redraw();
}

class MyBox : public Fl_Box {
public:
  MyBox(int X, int Y, int W, int H, const char *L)
    : Fl_Box(X,Y,W,H,L) {};
  virtual void draw() {
    Fl_Box::draw();
    // printf("MyBox::draw() - add_timeout(...)\n"); fflush(stdout);
    Fl::add_timeout(2.0, timer_cb, this);
  }
};

int main(int argc, char **argv) {
  Fl_Window *window = new Fl_Window(340,180);
  MyBox *box = new MyBox(20,40,300,100,"Hello, World!");
  box->box(FL_UP_BOX);
  box->labelfont(FL_BOLD+FL_ITALIC);
  box->labelsize(36);
  box->labeltype(FL_SHADOW_LABEL);
  window->end();
  window->show(argc, argv);
  return Fl::run();
}
